@@ -9,7 +9,7 @@ from urllib.parse import urljoin, urlparse
 import requests
 from bs4 import BeautifulSoup
 
-from autoscraper.utils import get_random_str, unique_hashable, unique_stack_list
+from autoscraper.utils import get_random_str, unique_hashable, unique_stack_list, ResultItem
 
 
 class AutoScraper(object):
@@ -153,7 +153,7 @@ class AutoScraper(object):
         children = [x for x in children if self._child_has_text(x, text, url)]
         return children
 
-    def build(self, url=None, wanted_list=None, html=None, request_args=None, update=False):
+    def build(self, url=None, wanted_list=None, wanted_dict=None, html=None, request_args=None, update=False):
         """
         Automatically constructs a set of rules to scrape the specified target[s] from a web page.
             The rules are represented as stack_list.
@@ -165,7 +165,12 @@ class AutoScraper(object):
 
         wanted_list: list, optional
             A list of needed contents to be scraped.
-                AutoScraper learns a set of rules to scrape these targets.
+                AutoScraper learns a set of rules to scrape these targets. If specified,
+                wanted_dict will be ignored.
+        
+        wanted_dict: dict, optional
+            A dict of needed contents to be scraped. Keys are aliases and values are list of target texts.
+                AutoScraper learns a set of rules to scrape these targets and sets its aliases.
 
         html: str, optional
             An HTML string can also be passed instead of URL.
@@ -191,16 +196,25 @@ class AutoScraper(object):
         if update is False:
             self.stack_list = []
 
-        wanted_list = [unicodedata.normalize("NFKD", w) for w in wanted_list]
+        if wanted_list:
+            wanted_dict = {'': wanted_list}
 
-        for wanted in wanted_list:
-            children = self._get_children(soup, wanted, url)
+        wanted_list = []
 
-            for child in children:
-                result, stack = self._get_result_for_child(child, soup, url)
-                result_list += result
-                self.stack_list.append(stack)
+        for alias, wanted_items in wanted_dict.items():
+            wanted_items = [unicodedata.normalize("NFKD", w) for w in wanted_items]
+            wanted_list += wanted_items
 
+            for wanted in wanted_items:
+                children = self._get_children(soup, wanted, url)
+
+                for child in children:
+                    result, stack = self._get_result_for_child(child, soup, url)
+                    stack['alias'] = alias
+                    result_list += result
+                    self.stack_list.append(stack)
+
+        result_list = [item.text for item in result_list]
         result_list = unique_hashable(result_list)
 
         if all(w in result_list for w in wanted_list):
@@ -258,9 +272,10 @@ class AutoScraper(object):
 
         return child.attrs[wanted_attr]
 
-    def _get_result_with_stack(self, stack, soup, url):
+    def _get_result_with_stack(self, stack, soup, url, **kwargs):
         parents = [soup]
         stack_content = stack['content']
+        contain_sibling_leaves = kwargs.get('contain_sibling_leaves', False)
         for index, item in enumerate(stack_content):
             children = []
             for parent in parents:
@@ -268,7 +283,7 @@ class AutoScraper(object):
                 if not found:
                     continue
 
-                if index == len(stack_content) - 1:
+                if not contain_sibling_leaves and index == len(stack_content) - 1:
                     idx = min(len(found) - 1, stack_content[index - 1][2])
                     found = [found[idx]]
 
@@ -278,11 +293,12 @@ class AutoScraper(object):
 
         wanted_attr = stack['wanted_attr']
         is_full_url = stack['is_full_url']
-        result = [self._fetch_result_from_child(i, wanted_attr, is_full_url, url) for i in parents]
-        result = [x for x in result if x]
+        result = [ResultItem(self._fetch_result_from_child(i, wanted_attr, is_full_url, url),
+                              getattr(i, 'child_index', 0)) for i in parents]
+        result = [x for x in result if x.text]
         return result
 
-    def _get_result_with_stack_index_based(self, stack, soup, url):
+    def _get_result_with_stack_index_based(self, stack, soup, url, **kwargs):
         p = soup.findChildren(recursive=False)[0]
         stack_content = stack['content']
         for index, item in enumerate(stack_content[:-1]):
@@ -293,32 +309,60 @@ class AutoScraper(object):
             idx = min(len(p) - 1, item[2])
             p = p[idx]
 
-        result = [self._fetch_result_from_child(p, stack['wanted_attr'], stack['is_full_url'], url)]
-        result = [x for x in result if x]
+        result = [ResultItem(self._fetch_result_from_child(
+            p, stack['wanted_attr'], stack['is_full_url'], url), getattr(p, 'child_index', 0))]
+        result = [x for x in result if x.text]
         return result
 
-    def _get_result_by_func(self, func, url, html, soup, request_args, grouped):
+    def _get_result_by_func(self, func, url, html, soup, request_args, grouped, 
+                            group_by_alias, unique, **kwargs):
         if not soup:
             soup = self._get_soup(url=url, html=html, request_args=request_args)
 
+        keep_order = kwargs.get('keep_order', False)
+
+        if group_by_alias or (keep_order and not grouped):
+            for index, child in enumerate(soup.findChildren()):
+                setattr(child, 'child_index', index)
+
         result_list = []
         grouped_result = defaultdict(list)
         for stack in self.stack_list:
             if not url:
                 url = stack.get('url', '')
 
-            result = func(stack, soup, url)
+            result = func(stack, soup, url, **kwargs)
 
-            if not grouped:
+            if not grouped and not group_by_alias:
                 result_list += result
                 continue
 
-            stack_id = stack['stack_id']
-            grouped_result[stack_id] += result
+            group_id = stack.get('alias', '') if group_by_alias else stack['stack_id']
+            grouped_result[group_id] += result
 
-        return dict(grouped_result) if grouped else unique_hashable(result_list)
+        return self._clean_result(result_list, grouped_result, grouped, group_by_alias, 
+                                  unique, keep_order)
 
-    def get_result_similar(self, url=None, html=None, soup=None, request_args=None, grouped=False):
+    @staticmethod
+    def _clean_result(result_list, grouped_result, grouped, grouped_by_alias, unique, keep_order):
+        if not grouped and not grouped_by_alias:
+            if keep_order:
+                result_list = sorted(result_list, key=lambda x: x.index)
+            result = [x.text for x in result_list]
+            if unique:
+                result = unique_hashable(result)
+            return result
+
+        for k, val in grouped_result.items():
+            if grouped_by_alias:
+                val = sorted(val, key=lambda x: x.index)
+            grouped_result[k] = [x.text for x in val]
+
+        return dict(grouped_result)
+
+    def get_result_similar(self, url=None, html=None, soup=None, request_args=None,
+                           grouped=False, group_by_alias=False, unique=True, keep_order=False,
+                           contain_sibling_leaves=False):
         """
         Gets similar results based on the previously learned rules.
 
@@ -339,16 +383,32 @@ class AutoScraper(object):
             If set to True, the result will be a dictionary with the rule_ids as keys
                 and a list of scraped data per rule as values.
 
+        group_by_alias: bool, optional, defaults to False
+            If set to True, the result will be a dictionary with the rule alias as keys
+                and a list of scraped data per alias as values.
+
+        unique: bool, optional, defaults to True
+            If set to True, will remove duplicates from returned result list. 
+
+        keep_order: bool, optional, defaults to False
+            If set to True, the results will be ordered as they are present on the web page.
+
+        contain_sibling_leaves: bool, optional, defaults to False
+            If set to True, the results will also contain the sibling leaves of the wanted elements.
+
         Returns:
         --------
         List of similar results scraped from the web page.
-        Dictionary if grouped=True.
+        Dictionary if grouped=True or group_by_alias=True.
         """
 
         func = self._get_result_with_stack
-        return self._get_result_by_func(func, url, html, soup, request_args, grouped)
+        return self._get_result_by_func(func, url, html, soup, request_args, grouped,
+                                         group_by_alias, unique, keep_order=keep_order,
+                                         contain_sibling_leaves=contain_sibling_leaves)
 
-    def get_result_exact(self, url=None, html=None, soup=None, request_args=None, grouped=False):
+    def get_result_exact(self, url=None, html=None, soup=None, request_args=None,
+                         grouped=False, group_by_alias=False, unique=True):
         """
         Gets exact results based on the previously learned rules.
 
@@ -369,16 +429,24 @@ class AutoScraper(object):
             If set to True, the result will be a dictionary with the rule_ids as keys
                 and a list of scraped data per rule as values.
 
+        group_by_alias: bool, optional, defaults to False
+            If set to True, the result will be a dictionary with the rule alias as keys
+                and a list of scraped data per alias as values.
+
+        unique: bool, optional, defaults to True
+            If set to True, will remove duplicates from returned result list. 
+
         Returns:
         --------
         List of exact results scraped from the web page.
-        Dictionary if grouped=True.
+        Dictionary if grouped=True or group_by_alias=True.
         """
 
         func = self._get_result_with_stack_index_based
-        return self._get_result_by_func(func, url, html, soup, request_args, grouped)
+        return self._get_result_by_func(func, url, html, soup, request_args, grouped, 
+                                        group_by_alias, unique)
 
-    def get_result(self, url=None, html=None, request_args=None, grouped=False):
+    def get_result(self, url=None, html=None, request_args=None, grouped=False, group_by_alias=False):
         """
         Gets similar and exact results based on the previously learned rules.
 
@@ -399,6 +467,10 @@ class AutoScraper(object):
             If set to True, the result will be dictionaries with the rule_ids as keys
                 and a list of scraped data per rule as values.
 
+        group_by_alias: bool, optional, defaults to False
+            If set to True, the result will be a dictionary with the rule alias as keys
+                and a list of scraped data per alias as values.
+
         Returns:
         --------
         Pair of (similar, exact) results.
@@ -406,8 +478,8 @@ class AutoScraper(object):
         """
 
         soup = self._get_soup(url=url, html=html, request_args=request_args)
-        similar = self.get_result_similar(url=url, soup=soup, grouped=grouped)
-        exact = self.get_result_exact(url=url, soup=soup, grouped=grouped)
+        similar = self.get_result_similar(url=url, soup=soup, grouped=grouped, group_by_alias=group_by_alias)
+        exact = self.get_result_exact(url=url, soup=soup, grouped=grouped, group_by_alias=group_by_alias)
         return similar, exact
 
     def remove_rules(self, rules):
@@ -442,6 +514,24 @@ class AutoScraper(object):
 
         self.stack_list = [x for x in self.stack_list if x['stack_id'] in rules]
 
+    def set_rule_aliases(self, rule_aliases):
+        """
+        Sets the specified alias for each rule
+
+        Parameters:
+        ----------
+        rule_aliases : dict
+            A dictionary with keys of rule_id and values of alias
+
+        Returns:
+        --------
+        None
+        """
+
+        id_to_stack = {stack['stack_id']: stack for stack in self.stack_list}
+        for rule_id, alias in rule_aliases.items():
+            id_to_stack[rule_id]['alias'] = alias
+
     def generate_python_code(self):
         # deprecated
         print('This function is deprecated. Please use save() and load() instead.')
